import { Slider } from "@bokehjs/models/widgets/slider";
export class ThrottledSlider extends Slider {
}
ThrottledSlider.__name__ = "ThrottledSlider";
ThrottledSlider.__module__ = "pyscivis.visualizer.models.throttled_slider";
//# sourceMappingURL=throttled_slider.js.map