.topcorner{
    position: absolute;
    top: 5px;
    right: 5px;
 }

.jstree-anchor {
    max-width: 100%;
    overflow-x: clip;
    text-overflow: ellipsis;
    display: inline-block;
}

.introjs-helperNumberLayer{
    display: none;
}

.loading {
    width: 100%;
    height: 100%;
    position: fixed;
    top: 0;
    right: 0;
    bottom: 0;
    left: 0;
    background-color: rgba(0,0,0,.5);
}

.loading-wheel {
    width: 20px;
    height: 20px;
    margin-top: -40px;
    margin-left: -40px;

    position: absolute;
    top: 50%;
    left: 50%;

    border-width: 30px;
    border-radius: 50%;
    -webkit-animation: spin 1s linear infinite;
}

.style-2 .loading-wheel {
    border-style: double;
    border-color: #ccc transparent;
}

@-webkit-keyframes spin {
    0% {
        -webkit-transform: rotate(0);
    }
    100% {
        -webkit-transform: rotate(-360deg);
    }
}

.loading-text {
    position: fixed !important;
    left: 50% !important;
    top: 60% !important;
    transform: translate(-50%, -50%) !important;
    font_size: large !important;
    color: #e4e4e4 !important;
}


/*
    Darkmode toggle adapted from: https://codepen.io/AllThingsSmitty/pen/MmxxOz/
    All credits go to Matt Smith @ https://github.com/AllThingsSmitty
*/

.switch {
  display: inline-block;
  height: 26px;
  position: relative;
  width: 40px;
}

.switch input {
  display:none;
}

.slider {
  background-color: gray;
  bottom: 0;
  cursor: pointer;
  left: 0;
  position: absolute;
  right: 0;
  top: 0;
  transition: .4s;
}

.slider:before {
  background-color: #15191C;
  bottom: 3px;
  content: "";
  height: 20px;
  left: 3px;
  position: absolute;
  transition: .4s;
  width: 20px;
}

input:checked + .slider:before {
  background-color: #E0E0E0;
  transform: translateX(14px);
}

.slider.round {
  border-radius: 26px;
}

.slider.round:before {
  border-radius: 50%;
}